Package: w3m
Version: 0.2.1-1
Section: text
Priority: optional
Architecture: i386
Depends: libc6 (>= 2.2.1-2), libgc5, libgpmg1 (>= 1.14-16), libncurses5 (>= 5.2.20010310-1)
Suggests: w3m-ssl (>= 0.2.1-1), mime-support, menu (>> 1.5), w3m-el
Provides: www-browser
Installed-Size: 1300
Maintainer: Fumitoshi UKAI <ukai@debian.or.jp>
Description: WWW browsable pager with excellent tables/frames support
 w3m is a text-based World Wide Web browser with IPv6 support.
 It features excellent support for tables and frames. It can be used
 as a standalone file pager, too.
 .
  * You can follow links and/or view images in HTML.
  * Internet message prewview mode, you can browse HTML mail.
  * You can follow links in plain text if it includes URL forms.
 .
 For more information,
 see http://ei5nazha.yz.yamagata-u.ac.jp/~aito/w3m/eng/index.html
